require 'optparse'
require 'ostruct'

module SastBox
  module OptParser
    def parse_opts(args)
      @opts = OpenStruct.new
      @opts.verbose     = false
      @opts.info        = false
      @opts.color       = true
      @opts.diff_hashes = []
      @opts.diff_quick  = false
      @opts.timeout     = 200 * 60

      opt_parser = OptionParser.new do |opts|
        opts.on('-c', '--codebase=CODEBASE', 'Codebase to be scanned') do |codebase|
          @opts.codebase = codebase
        end

        opts.on('-o', '--output=OUTPUT', 'Output path to save SARIF result') do |output|
          @opts.output = output
        end

        opts.on("-v", '--[no-]verbose', 'Run verbosely') do |v|
          @opts.verbose = v
        end

        opts.on('-t', '--timeout=TIMEOUT', Integer, "Control timeout (default: #{@opts.timeout/60} mins per scanner)") do |timeout|
          @opts.timeout = timeout * 60
        end

        opts.on("-i", '--info', 'Info about the scanner') do
          @opts.info = true
        end

        opts.on('-n', '--[no-]color', 'Enable/disable coloring') do |v|
          @opts.color = v
        end
      end

      opt_parser.parse!(args)
      @opts
    end
  end
end
