require 'set'

module SastBox
  module Cwe

    def cwe_found?(issue, patterns, cwe)
      patterns.each do |pattern|
        @alternative_titles.each do |title|
          if title.include? pattern
            issue[:cwe_id] = cwe
            return true
          end
        end
      end
      return false
    end

    def detected_sql_injection?(issue)
      patterns = [
        'sql injection',
        'sqlinj',
        'sqli',
        'sql inj'
      ]
      cwe_found?(issue, patterns, SastBox::Cwe::SQL_INJECTION)
    end

    def detected_xss?(issue)
      patterns = [
        'xss',
        'cross-site scripting',
        'cross site scripting',
        'html injection'
      ]
      cwe_found?(issue, patterns, SastBox::Cwe::XSS)
    end

    def detected_cmd_injection?(issue)
      patterns = [
        'command injection',
        'command execution',
        'cmd injection',
        'cmd execution',
        'cmd exec',
        'shell injection',
        'shell metacharacters'
      ]
      cwe_found?(issue, patterns, SastBox::Cwe::OS_COMMAND_INJECTION)
    end

    def detected_code_injection?(issue)
      patterns = [
        'code injection',
        'code execution',
        'code exec',
        'code inj'
      ]
      cwe_found?(issue, patterns, SastBox::Cwe::CODE_INJECTION)
    end

    def detected_session_fixation?(issue)
      patterns = [
        'session fixation',
      ]
      cwe_found?(issue, patterns, SastBox::Cwe::SESSION_FIXATION)
    end

    def detected_csrf?(issue)
      patterns = [
        'csrf',
        'xsrf',
        'cross site request forgery',
        'session riding',
        'cross site reference forgery',
      ]
      cwe_found?(issue, patterns, SastBox::Cwe::CSRF)
    end

    def detected_deserialization?(issue)
      patterns = [
        'deserializ',
        'unmarshaling',
        'unpickling',
        'php object injection'
      ]
      cwe_found?(issue, patterns, SastBox::Cwe::DESERIALIZATION_OF_UNTRUSTED_DATA)
    end

    def detected_path_traversal?(issue)
      patterns = [
        'path traversal',
        'traversal',
        'pathtraversal'
      ]
      cwe_found?(issue, patterns, SastBox::Cwe::PATH_TRAVERSAL)
    end

    def detected_hardcoded_password?(issue)
      patterns = [
        'hard-coded'
      ]
      cwe_found?(issue, patterns, SastBox::Cwe::HARD_CODED_PASSWORD)
    end

    def detected_null_ptr_deref?(issue)
      patterns = [
        'null pointer deref'
      ]
      cwe_found?(issue, patterns, SastBox::Cwe::NULL_POINTER_DEREFERENCE)
    end

    def detected_broken_crypto?(issue)
      patterns = [
        'weak cipher',
        'weak crypto',
        'insecure cipher',
        'insecure crypto',
        'insecure encryption',
        'broken cipher',
        'broken crypto',
        'weak hash',
        'insecure hash',
        'broken hash',
      ]
      cwe_found?(issue, patterns, SastBox::Cwe::BROKEN_CRYPTO)
    end

    def detected_improper_authorization?(issue)
      patterns = [
        'improper authorization',
        'no authorization',
        'broken authorization',
      ]
      cwe_found?(issue, patterns, SastBox::Cwe::IMPROPER_AUTHORIZATION)
    end

    def detected_improper_authentication?(issue)
      patterns = [
        'improper authentication',
        'no authentication',
        'broken authentication',
      ]
      cwe_found?(issue, patterns, SastBox::Cwe::IMPROPER_AUTHENTICATION)
    end

    def detected_improper_input_validation?(issue)
      patterns = [
        'input validation',
        'data validation',
      ]
      cwe_found?(issue, patterns, SastBox::Cwe::IMPROPER_INPUT_VALIDATION)
    end

    def detected_unrestricted_file_upload?(issue)
      patterns = [
        'unrestricted upload',
        'unrestricted file upload',
      ]
      cwe_found?(issue, patterns, SastBox::Cwe::UNRESTRICTED_UPLOAD_OF_FILE_WITH_DANGEROUS_TYPE)
    end

    def cwe_start_heuristics(issue)
      alternative_titles(issue)
      issue[:cwe_id] = SastBox::Cwe::UNDEF
      return if detected_sql_injection?(issue)
      return if detected_xss?(issue)
      return if detected_cmd_injection?(issue)
      return if detected_code_injection?(issue)
      return if detected_session_fixation?(issue)
      return if detected_csrf?(issue)
      return if detected_deserialization?(issue)
      return if detected_path_traversal?(issue)
      return if detected_hardcoded_password?(issue)
      return if detected_null_ptr_deref?(issue)
      return if detected_broken_crypto?(issue)
      return if detected_improper_authorization?(issue)
      return if detected_improper_authentication?(issue)
      return if detected_improper_input_validation?(issue)
      return if detected_unrestricted_file_upload?(issue)
    end

    def alternative_titles(issue)
      @alternative_titles = Set.new
      @alternative_titles << issue[:title].downcase

      @alternative_titles << @alternative_titles.first.split('_').join(' ')
      @alternative_titles << @alternative_titles.first.split('-').join(' ')
      @alternative_titles << @alternative_titles.first.gsub("hard coded", "hard-coded")
      @alternative_titles << @alternative_titles.first.gsub("hardcoded", "hard-coded")
      @alternative_titles
    end

    def guess_cwe(issue)
      if issue.key?(:cwe_id)
        cwe_start_heuristics(issue) if [SastBox::Cwe::UNDEF, -1, nil].include?(issue[:cwe_id])
      else
        cwe_start_heuristics(issue)
      end
    end

  end
end